import torchnorms.tconorms.base
import torchnorms.tconorms.classic
import torchnorms.tconorms.dombi
import torchnorms.tconorms.dubois
import torchnorms.tconorms.hamacher
import torchnorms.tconorms.ss
import torchnorms.tconorms.weber
import torchnorms.tconorms.yager
